package com.corba.mnq.ns;

import java.io.File;
import java.io.FileInputStream;
import java.util.List;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.omg.CosNaming.Binding;
import org.omg.CosNaming.BindingIteratorHolder;
import org.omg.CosNaming.BindingListHolder;
import org.omg.CosNaming.BindingType;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;

import com.corba.mnq.main.CorbaMNQ;
import com.corba.mnq.tool.CorbaFacility;

public class NSConsoleUtil {

	private static NamingContextExt rootContext = null;

	private static String getIorFromFile(String file) {
		String ior = null;
		try {
			FileInputStream fis = new FileInputStream(file);
			String str = "";
			while (fis.available() != 0) {
				byte[] bA = new byte[fis.available()];
				int numRead = fis.read(bA);
				if (numRead > 0) {
					str += new String(bA, 0, numRead);
				}
				// continue
			}
			ior = str;
			fis.close();
			// java.io.BufferedReader in =
			// new java.io.BufferedReader(new java.io.FileReader( file
			// ) );
			// ior = in.readLine();
			// while (ior.indexOf("IOR:") != 0)
			// ior = in.readLine();
			// in.close();
		} catch (java.io.IOException io) {
			io.printStackTrace();
		}
		return ior.trim();
	}

	public static void connectNS() {
		rootContext = null;
		try {
			String ior = getIorFromFile(CorbaMNQ.ossDir + File.separator
					+ "namingService.ior");
			if (ior == null) {
				System.out.println("Invalid IOR file!");
			}
			rootContext = NamingContextExtHelper.narrow(CorbaFacility.orb
					.string_to_object(ior));

		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Could not find name service!");
			return;
		}

		if (rootContext == null) {
			System.out.println("Could not find name service!");
			return;
		}
	}

	public static void list(String name) {
		try {
			NameComponent[] ncs = strToNC(name);
			org.omg.CORBA.Object obj = rootContext.resolve(ncs);
			if (obj instanceof org.omg.CosNaming.NamingContextExt
					|| obj._is_a("IDL:omg.org/CosNaming/NamingContextExt:1.0")) {
				list(NamingContextExtHelper.narrow(obj), name);
			} else {
				System.out.println("This is a Corba Object.");
				System.out.println(CorbaFacility.orb.object_to_string(obj));
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public static void listAll(String name) {
		try {
			NameComponent[] ncs = strToNC(name);
			org.omg.CORBA.Object obj = rootContext.resolve(ncs);
			if (obj instanceof org.omg.CosNaming.NamingContextExt
					|| obj._is_a("IDL:omg.org/CosNaming/NamingContextExt:1.0")) {
				listAll(NamingContextExtHelper.narrow(obj), 0, name);
			} else {
				System.out.println("This is a Corba Object.");
				System.out.println(CorbaFacility.orb.object_to_string(obj));
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public static void listRoot() {
		listAll(rootContext, 0, "");
	}

	public static void list(NamingContextExt nce, String prefix) {
		try {
			BindingListHolder blsoh = new BindingListHolder();
			BindingIteratorHolder bioh = new BindingIteratorHolder();

			nce.list(NSTree.MAX_BIND, blsoh, bioh);
			Binding[] bindings = blsoh.value;

			// set up lists of object bindings and subcontext bindings

			for (int i = 0; i < bindings.length; i++) {
				if (bindings[i].binding_type == BindingType.ncontext) {
					System.out.println("ContextNode: " + prefix
							+ ncToString(bindings[i].binding_name));
				} else {
					System.out.println("ObjectNode : " + prefix
							+ ncToString(bindings[i].binding_name));
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * update the content of this node and all its children
	 */

	public static void listAll(NamingContextExt nce, int n, String prefix) {
		try {
			BindingListHolder blsoh = new BindingListHolder();
			BindingIteratorHolder bioh = new BindingIteratorHolder();

			nce.list(NSTree.MAX_BIND, blsoh, bioh);
			Binding[] bindings = blsoh.value;

			// set up lists of object bindings and subcontext bindings

			for (int i = 0; i < bindings.length; i++) {
				if (bindings[i].binding_type == BindingType.ncontext) {
					System.out.println(nTab(n) + "ContextNode: " + prefix
							+ ncToString(bindings[i].binding_name));
					listAll(NamingContextExtHelper.narrow(nce
							.resolve(bindings[i].binding_name)), n + 1, prefix
							+ ncToString(bindings[i].binding_name));

				} else {
					System.out.println(nTab(n) + "ObjectNode : " + prefix
							+ ncToString(bindings[i].binding_name));
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static String nTab(int len) {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < len; i++) {
			sb.append("   ");
		}

		return sb.toString();
	}

	private static String ncToString(NameComponent[] ncs) {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < ncs.length; i++) {
			sb.append("/");
			NameComponent nc = ncs[i];
			sb.append(nc.id).append(".").append(nc.kind);
		}

		return sb.toString();
	}

	private static NameComponent[] strToNC(String str) {
		NameComponent[] ncs = null;

		List nclist = new Vector();

		// group 1--id, group 2--kind
		String boxVal_dcl = "/([^/\\.]*)\\.([^/\\.]*)";

		Pattern pNc = Pattern.compile(boxVal_dcl, Pattern.DOTALL
				+ Pattern.MULTILINE);
		Matcher mNc = pNc.matcher(str);
		while (mNc.find()) {
			String id = mNc.group(1);
			String kind = mNc.group(2);
			nclist.add(new NameComponent(id, kind));
		}

		ncs = new NameComponent[nclist.size()];

		for (int i = 0; i < nclist.size(); i++) {
			ncs[i] = (NameComponent) nclist.get(i);
		}

		return ncs;
	}

}
