
package com.corba.mnq.ns;

import java.awt.Component;
import java.awt.Frame;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;

import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.table.JTableHeader;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import com.corba.mnq.main.CorbaMNQ;

public class NSHandler extends WindowAdapter implements ActionListener,
		MouseListener, KeyListener {
	JTree tree;

	NSTable nstab;

	Component frame;

	JDialog dlg;

	JTextField editName;

	JPopupMenu popup = new JPopupMenu();

	JMenuItem bindContext = new JMenuItem("BindNewContext");

	JMenuItem exportIor = new JMenuItem("ExportIOR");

	// JMenuItem sortCol = new JMenuItem("Sort");
	JMenuItem bindObject = new JMenuItem("Bind Object");

	JMenuItem unbind = new JMenuItem("Unbind name");

	JMenuItem connectNsMenu = new JMenuItem("ConnectNamingService");

	JMenuItem refresh = new JMenuItem("Refresh");

	JMenuItem copyTxt = new JMenuItem("Copy");

	ContextNode currentNode = null;

	public void reset(JTree tr, NSTable tab) {
		tree = tr;
		nstab = tab;
		((NSTree) tree).update();
	}

	private void copy() {
		int col = nstab.getSelectedColumn();
		int row = nstab.getSelectedRow();
		Object o = nstab.getValueAt(row, col);
		Clipboard clipboard = frame.getToolkit().getSystemClipboard();
		StringSelection textInfoSelected = new StringSelection(o.toString());
		clipboard.setContents(textInfoSelected, null);
	}

	public NSHandler(Component fr, JTree tr) {
		frame = fr;
		tree = tr;

		bindContext.addActionListener(this);
		bindObject.addActionListener(this);
		unbind.addActionListener(this);
		connectNsMenu.addActionListener(this);
		exportIor.addActionListener(this);
		refresh.addActionListener(this);
		copyTxt.addActionListener(this);
	}

	public void actionPerformed(ActionEvent e) {

		if (e.getActionCommand().equals("Unbind name")) {
			((NSTree) tree).unbind();
		} else if (e.getActionCommand().equals("Bind Object")) {
			ObjectDialog dialog = new ObjectDialog((Frame) frame);
			// dialog.pack();
			// dialog.show();

			if (dialog.isOk) {
				try {
					((NSTree) tree).bindObject(dialog.getName(), dialog
							.getIOR().trim(), dialog.isRebind());
				} catch (org.omg.CORBA.UserException ue) {
					JOptionPane.showMessageDialog(frame,
							ue.getClass().getName()
									+ (ue.getMessage() != null ? (":" + ue
											.getMessage()) : ""), "Exception",
							JOptionPane.INFORMATION_MESSAGE);
				}
			}
		} else if (e.getActionCommand().equals("BindNewContext")) {
			try {
				String contextName = JOptionPane.showInputDialog(frame,
						"Name of the new context", "BindNewContext",
						JOptionPane.QUESTION_MESSAGE);

				// check if user input is okay or if CANCEL was hit
				if (contextName != null && contextName.length() > 0)
					((NSTree) tree).bind(contextName);
			} catch (org.omg.CORBA.UserException ue) {
				JOptionPane.showMessageDialog(frame, ue.getClass().getName()
						+ (ue.getMessage() != null ? (":" + ue.getMessage())
								: ""), "Exception",
						JOptionPane.INFORMATION_MESSAGE);
			}
		} else if (e.getActionCommand().equals("ConnectNamingService")) {
			NSUtil.reset();
		} else if (e.getActionCommand().equals("Refresh")) {
			if (tree instanceof NSTree)
				((NSTree) tree).update();
		} else if (e.getActionCommand().equals("Copy")) {
			copy();
		} else if (e.getActionCommand().equals("ExportIOR")) {
			String ior = nstab.getIOR();
			if (ior == null) {
				JOptionPane.showMessageDialog(frame,
						"Cannot export IOR for this object", "Operation tip",
						JOptionPane.OK_OPTION);
			}
			try {
				File iorDir = new File(CorbaMNQ.projectDir + File.separator
						+ "ior");
				if (!iorDir.exists()) {
					iorDir.mkdir();
				}
				iorDir = new File(CorbaMNQ.projectDir + File.separator + "ior"
						+ File.separator + "ucs");
				if (!iorDir.exists()) {
					iorDir.mkdir();
				}
				// IOR File
				File dir = new File(CorbaMNQ.projectDir + File.separator
						+ "ior" + File.separator + "ucs");
				JFileChooser dlg = new JFileChooser(dir);
				// JFileChooser dlg=new JFileChooser();
				if (dlg.showSaveDialog(frame) == JFileChooser.APPROVE_OPTION) {
					String fname = dlg.getSelectedFile().getPath();

					PrintWriter out = new PrintWriter(new FileOutputStream(
							fname), true);
					out.println(ior);
					out.close();
				}
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}

	}

	/**
	 * @param k
	 *            java.awt.event.KeyEvent
	 */

	public void keyPressed(KeyEvent k) {
	}

	/**
	 * @param k
	 *            java.awt.event.KeyEvent
	 */

	public void keyReleased(KeyEvent k) {
		if (k.getComponent() == tree) {
			if (k.getKeyCode() == KeyEvent.VK_DELETE)
				((NSTree) tree).unbind();
		} else if (k.getComponent() == nstab) {
			if (k.getKeyCode() == KeyEvent.VK_DELETE)
				nstab.unbind();
		}
	}

	public void keyTyped(KeyEvent k) {
	}

	public void mouseClicked(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
	}

	/**
	 * opens pop-up menu or displays context node content
	 */

	public void mouseReleased(MouseEvent e) {
		if (e.getComponent() == tree) {
			// on Solaris, the right mouse button somehow seems not be a popup
			// trigger, so we
			// accept mouse 3 explicitly
			TreePath path = tree.getPathForLocation(e.getX(), e.getY());
			if (path != null) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) path
						.getPathComponent(path.getPathCount() - 1);
				currentNode = (ContextNode) node.getUserObject();
			}
			if (e.isPopupTrigger()
					|| e.getModifiers() == java.awt.event.InputEvent.BUTTON3_MASK) {
				popup.removeAll();

				if (path != null) {
					popup.add(refresh);
					popup.add(bindContext);
					popup.add(bindObject);
					popup.add(unbind);
				} else {
					popup.add(connectNsMenu);
				}
				popup.pack();
				popup.show(tree, e.getX(), e.getY());
			} else {
				if (path != null) {
					DefaultMutableTreeNode node = (DefaultMutableTreeNode) path
							.getPathComponent(path.getPathCount() - 1);
					((ContextNode) node.getUserObject()).display();
				}
			}
		} else if (e.getComponent() == nstab) {
			if (e.isPopupTrigger()
					|| e.getModifiers() == java.awt.event.InputEvent.BUTTON3_MASK) {
				if (nstab.canExportIor()) {
					popup.removeAll();
					popup.add(exportIor);
					int col = nstab.getSelectedColumn();
					int row = nstab.getSelectedRow();
					if (col >= 0 && row >= 0) {
						popup.add(copyTxt);
					}
					popup.pack();
					popup.show(nstab, e.getX(), e.getY());
				}
			}
		} else if (e.getComponent() == nstab.getTableHeader()) {
			if (e.getClickCount() == 2) {
				JTableHeader h = nstab.getTableHeader();
				int col = h.columnAtPoint(e.getPoint());
				if (col >= 0) { // sort
					currentNode.sortBy(col);
				}
			}
		}
	}
}
