// **********************************************************************
//
// Copyright (c) 2004-2009
//
// Regular Triangle Team (Zhang Guoyou -- Wei Chaojiong -- Zhang Dagang)
//
// All Rights Reserved
//
// **********************************************************************
package org.jacorb.orb.dynany;

/*
 *        JacORB  - a free Java ORB
 *
 *   Copyright (C) 1997-2004  Gerald Brose.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import org.apache.avalon.framework.logger.Logger;
import org.jacorb.orb.CDRInputStream;
import org.jacorb.orb.CDROutputStream;
import org.jacorb.orb.TypeCode;
import org.omg.DynamicAny.DynAnyPackage.*;
import org.omg.DynamicAny.*;

/**
 * CORBA DynValue
 * 
 * @author Nick Cross
 * @version $Id: DynValue.java,v 1.5 2006/06/20 07:14:36
 *          alphonse.bendt Exp $
 */

public final class DynValue extends DynAny implements org.omg.DynamicAny.DynValue {

    /**
     * type: long name: serialVersionUID
     */
    private static final long serialVersionUID = -3282757580882273964L;

    private org.omg.DynamicAny.NameValuePair[] members;

    private DynValue() {
        super(null, null, null);
        isNull = true;
    }

    DynValue(org.omg.DynamicAny.DynAnyFactory dynFactory, org.omg.CORBA.TypeCode type,
            org.omg.CORBA.ORB orb, Logger logger) throws TypeMismatch {
        super(dynFactory, orb, logger);

        org.omg.CORBA.TypeCode _type = TypeCode.originalType(type);

        if (_type.kind().value() != org.omg.CORBA.TCKind._tk_value) { throw new TypeMismatch(); }

        typeCode = _type;
        isNull = false;

        try {
            /* initialize position for all except empty exceptions */
            if (!isEmptyEx()) {
                pos = 0;
            }

            limit = typeCode.member_count();
            members = new NameValuePair[limit];
            for (int i = 0; i < limit; i++) {
                org.omg.CORBA.TypeCode _tc = TypeCode.originalType(typeCode.member_type(i));
                members[i] = new NameValuePair(typeCode.member_name(i), dynFactory
                        .create_dyn_any_from_type_code(_tc).to_any());

            }
        } catch (org.omg.DynamicAny.DynAnyFactoryPackage.InconsistentTypeCode e) {
            throw unexpectedException(e);
        } catch (org.omg.CORBA.TypeCodePackage.BadKind e) {
            throw unexpectedException(e);
        } catch (org.omg.CORBA.TypeCodePackage.Bounds e) {
            throw unexpectedException(e);
        }
    }

    public java.lang.String current_member_name() throws TypeMismatch, InvalidValue {
        checkDestroyed();

        if (isEmptyEx()) { throw new TypeMismatch(); }
        if (pos == -1) { throw new InvalidValue(); }
        return members[pos].id;
    }

    public org.omg.CORBA.TCKind current_member_kind() throws TypeMismatch, InvalidValue {
        checkDestroyed();

        if (isEmptyEx()) { throw new TypeMismatch(); }
        if (pos == -1) { throw new InvalidValue(); }
        return members[pos].value.type().kind();
    }

    public NameValuePair[] get_members() {
        checkDestroyed();
        return members;
    }

    public void set_members(NameValuePair[] nvp) throws InvalidValue, TypeMismatch {
        checkDestroyed();
        if (nvp.length != limit) { throw new org.omg.DynamicAny.DynAnyPackage.InvalidValue(); }

        for (int i = 0; i < limit; i++) {
            if (!nvp[i].value.type().equivalent(members[i].value.type())) { throw new org.omg.DynamicAny.DynAnyPackage.TypeMismatch(); }

            if (!(nvp[i].id.equals("") || nvp[i].id.equals(members[i].id))) { throw new org.omg.DynamicAny.DynAnyPackage.TypeMismatch(); }
        }
        members = nvp;
    }

    public org.omg.DynamicAny.NameDynAnyPair[] get_members_as_dyn_any() {
        checkDestroyed();
        NameDynAnyPair[] result = new NameDynAnyPair[limit];
        try {
            for (int i = 0; i < limit; i++) {
                result[i] = new NameDynAnyPair(members[i].id, dynFactory
                        .create_dyn_any(members[i].value));
            }

            return result;
        } catch (org.omg.DynamicAny.DynAnyFactoryPackage.InconsistentTypeCode e) {
            throw unexpectedException(e);
        }
    }

    public void set_members_as_dyn_any(org.omg.DynamicAny.NameDynAnyPair[] nvp)
            throws TypeMismatch, InvalidValue {
        checkDestroyed();
        if (nvp.length != limit) { throw new org.omg.DynamicAny.DynAnyPackage.InvalidValue(); }

        for (int i = 0; i < limit; i++) {
            if (!nvp[i].value.type().equivalent(members[i].value.type())) { throw new org.omg.DynamicAny.DynAnyPackage.TypeMismatch(); }

            if (!(nvp[i].id.equals("") || nvp[i].id.equals(members[i].id))) { throw new org.omg.DynamicAny.DynAnyPackage.TypeMismatch(); }

        }
        members = new NameValuePair[nvp.length];
        for (int i = 0; i < limit; i++) {
            members[i] = new NameValuePair(nvp[i].id, nvp[i].value.to_any());
        }
    }

    public void set_to_null() {
        // TODO Auto-generated method stub
        isNull = true;
        members = null;
    }

    public void set_to_value() {
        // TODO Auto-generated method stub
        if (isNull) {
            isNull = false;
            // the rest is done lazily
        }
        // else: there is nothing to do
    }

    public boolean is_null() {
        // TODO Auto-generated method stub
        return isNull;
    }

    private boolean isEmptyEx() {
        return false;
    }

    public org.omg.CORBA.Any to_any() {
        checkDestroyed();
        org.omg.CORBA.Any out_any = orb.create_any();
        out_any.type(type());

        final CDROutputStream out = new CDROutputStream(orb);

        try {
            out.write_value_header(new String[] { typeCode.id() });
            out.start_chunk();
                        
            for (int i = 0; i < members.length; i++) {
                out.write_value(members[i].value.type(), members[i].value.create_input_stream());
            }
            
            out.end_chunk();

            final CDRInputStream in = new CDRInputStream(orb, out.getBufferCopy());
            try {
                out_any.read_value(in, type());
                return out_any;
            } finally {
                in.close();
            }
        } catch(Exception ex) {
            ex.printStackTrace();
            return null;
        }
        finally {
            out.close();
        }
    }

    protected boolean isNull;
}
